// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@infoscreen.com, or by
// paper mail at:
//
// Tim Freeman
// 655 S. FairOaks Ave., Apt B-316
// Sunnyvale, CA 94086
//

#ifndef __Factory_h__
#include "Factory.h"
#endif

#ifndef __String_h__
#include "String.h"
#endif

#ifndef __SP_h__
#include "SP.h"
#endif

#ifndef __Configuration_h__
#include "Configuration.h"
#endif

#ifndef __MemoryUtil_h__
#include "MemoryUtil.h"
#endif

#ifndef __ActionConfiguration_h__
#include "ActionConfiguration.h"
#endif

#ifndef __Configurable_h__
#include "Configurable.h"
#endif

#ifndef __TopLevel_h__
#include "TopLevel.h"
#endif

#ifndef __SelectionManager_h__
#include "SelectionManager.h"
#endif

#ifndef __SceneGraph_h__
#include "SceneGraph.h"
#endif

#ifndef __Dynavec_h__
#include "Dynavec.h"
#endif

#ifndef __FactoryTable_h__
#include "FactoryTable.h"
#endif

#ifndef __Action_h__
#include "Action.h"
#endif

#ifndef __LinkManager_h__
#include "LinkManager.h"
#endif

namespace {
  class DeleteSelection
    : public Factory
  {
  public:
    DeleteSelection ()
      : Factory ("DeleteSelection")
    {}
    SP<Configuration> defaultConfiguration () const {
      return NEW (ActionConfiguration ());
    }
    SP<Configurable> makeIt (const SP<Configuration> c) const {
      SP<ActionConfiguration> conf =
	dynamic_cast <ActionConfiguration *> (&*c);
      assert (conf);
      SP<TopLevel> top = conf->getTopLevel ();
      SP<SceneGraph> sg = top->getSceneGraph ();
      SP<LinkManager> lm = sg->getLinkManager ();
      Dynavec <int> selection;
      Dynavec <int> endSelection;
      SelectionManager::getSelection (selection);
      SP<Action> result = NEW (Action ());
      if (0 == selection.size ()) {
	result->setProblem ("Nothing was deleted because nothing "
			    "was selected.");
      } else {
	for (int i = 0; i < selection.size(); i++) {
	  const int *links;
	  int numlinks;
	  lm->getLinks (selection [i], numlinks, links);
	  for (int j = 0; j < numlinks; j++) {
	    const int thisLink = links [j];
	    bool willDelete = false;
	    if (-1 == thisLink) {
	      willDelete = true;
	    } else {
	      for (int k = 0; k < selection.size (); k++) {
		if (thisLink == selection [k]) {
		  willDelete = true;
		  break;
		}
	      }
	    }
	    if (! willDelete) {
	      endSelection.push (thisLink);
	    }
	  }
	  sg->deleteObject (selection [i]);
	}
	SelectionManager::setSelection (sg, endSelection);
      }
      return &*result;
    }
  };
  const bool useless =
  (FactoryTable::store ("Action", NEW (DeleteSelection ())),
   true);
}
